from __future__ import absolute_import
import re

from ..exceptions import ApiException


class BaseRequest(object):
    def __init__(self, client_id=None, signature_key=None, client_api_key=None, live_mode=None):
        if client_id is not None:
            self._client_id = client_id
        if signature_key is not None:
            self._signature_key = signature_key
        if client_api_key is not None:
            self._client_api_key = client_api_key
        if live_mode is not None:
            self._live_mode = live_mode    

    @property
    def client_id(self):
        return self._client_id

    @client_id.setter
    def client_id(self, client_id):

        if client_id is not None and len(client_id) > 50:
            raise ApiException(status=400, reason="Invalid value for `id`, length must be less than or equal to `50`")
        if client_id is not None and len(client_id) < 0:
            raise ApiException(status=400, reason="Invalid value for `id`, length must be greater than or equal to `0`")
        if client_id is not None and not re.search('^[A-Za-z0-9-_]{0,50}$', client_id):
            raise ApiException(status=400,
                               reason="Invalid value for `id`, must be a follow pattern or equal to `/^[A-Za-z0-9-_]{0,50}$/`")

        self._client_id = client_id

    @property
    def signature_key(self):
        return self._signature_key

    @signature_key.setter
    def signature_key(self, signature_key):

        if signature_key is not None and len(signature_key) > 64:
            raise ApiException(status=400, reason="Invalid value for `id`, length must be less than or equal to `50`")
        if signature_key is not None and len(signature_key) < 0:
            raise ApiException(status=400, reason="Invalid value for `id`, length must be greater than or equal to `0`")
        if signature_key is not None and not re.search('^[A-Za-z0-9]{0,64}$', signature_key):
            raise ApiException(status=400,
                               reason="Invalid value for `id`, must be a follow pattern or equal to `/^[A-Za-z0-9-_]{0,50}$/`")

        self._signature_key = signature_key

    @property
    def client_api_key(self):
        return self._client_api_key

    @client_api_key.setter
    def client_api_key(self, client_api_key):
        self._client_api_key = client_api_key

    @property
    def live_mode(self):
        return self._live_mode

    @live_mode.setter
    def live_mode(self, live_mode):
        self._live_mode = live_mode    
