from __future__ import absolute_import
import base64
import hashlib
import hmac
import json

from six import iterkeys

from ..exceptions import ApiException
from .logger_util import LoggerUtil
from ..constants import Constants
from ..enums import CheckoutHeaders



class AuthenticationUtil(object):
    logger = LoggerUtil.setup_logger(__name__)
    ENCODE_TYPE = 'utf-8'

    @staticmethod
    def computeHMACSHA256HASH(path, method, query_params,
                              header_params, body, signature_key):
        try:
            path = path[path.index(Constants.APP_URL_BASE):]
            seperator = ":"
            params = []
            params.append(method.upper().strip(' '))
            params.append(path.strip(' '))
            params.append(AuthenticationUtil.get_query_params_string(query_params))
            params.append(AuthenticationUtil.get_header_params_string(header_params))
            body_text = json.dumps(body)
            #AuthenticationUtil.logger.info("body_text: {}".format(body_text))
            if body_text and "null" != body_text:
                params.append(body_text)
                param_string = seperator.join(params)
            else:
                param_string = seperator.join(params) + seperator

            #AuthenticationUtil.logger.info("param_string: {}".format(param_string))
            hmac_sha_256_hash = AuthenticationUtil.decrypt_encoded_data(secret=signature_key, message=param_string)
            #AuthenticationUtil.logger.info("hmac_sha_256_hash: {}".format(hmac_sha_256_hash))
            return hmac_sha_256_hash
        except BaseException as e:
            AuthenticationUtil.logger.error("exception occured: {}".format(e))
            raise ApiException(
                status=0,
                reason="Failed to compute HMACSHA256 hash {}".format(e)
            )

    @staticmethod
    def get_query_params_string(query_params):
        try:
            filtered_query_params = {}
            for param in query_params:
                #AuthenticationUtil.logger.info("param: {}".format(param))
                filtered_query_params[param[0]] = param[1]

            #AuthenticationUtil.logger.info("filtered_query_params: {}".format(filtered_query_params))
            queryString = ""
            i = 0
            for key in sorted(iterkeys(filtered_query_params)):
                if key and filtered_query_params[key]:
                    value = filtered_query_params[key]

                    #AuthenticationUtil.logger.info("key: {}".format(key))
                    #AuthenticationUtil.logger.info("value: {}".format(value))

                    if i != len(filtered_query_params) - 1:
                        queryString += key + "=" + value + "&"
                    else:
                        queryString += key + "=" + value
                i = i + 1
            #AuthenticationUtil.logger.info("queryString: {}".format(queryString))
            return queryString
        except BaseException as e:
            AuthenticationUtil.logger.error("exception occured: {}".format(e))
            raise ApiException(
                status=0,
                reason="Failed to get_query_params_string {}".format(e)
            )

    @staticmethod
    def get_header_params_string(header_params):
        try:
            filtered_header_params = {}
            for key in sorted(iterkeys(header_params)):
                if CheckoutHeaders.is_valid_checkout_header(key):
                    filtered_header_params[key] = header_params[key]

            #AuthenticationUtil.logger.info("filtered_header_params: {}".format(filtered_header_params))
            headerString = ""
            i = 0
            for key in sorted(iterkeys(filtered_header_params)):
                if key and filtered_header_params[key]:
                    value = filtered_header_params[key]

                    #AuthenticationUtil.logger.info("key: {}".format(key))
                    #AuthenticationUtil.logger.info("value: {}".format(value))

                    if i != len(filtered_header_params) - 1:
                        headerString += key + "=" + value + "&"
                    else:
                        headerString += key + "=" + value
                i = i + 1
            #AuthenticationUtil.logger.info("headerString: {}".format(headerString))
            return headerString
        except BaseException as e:
            AuthenticationUtil.logger.error("exception occured: {}".format(e))
            raise ApiException(
                status=0,
                reason="Failed to get_header_params_string {}".format(e)
            )

    @staticmethod
    def decrypt_encoded_data(secret, message):
        try:
            dig = hmac.new(secret.encode(AuthenticationUtil.ENCODE_TYPE), msg=message.encode(AuthenticationUtil.ENCODE_TYPE), digestmod=hashlib.sha256).digest()
            base64_encoded_data = base64.b64encode(dig).decode()
            #AuthenticationUtil.logger.info("base64_encoded_data: {}".format(base64_encoded_data))
            return base64_encoded_data
        except BaseException as e:
            AuthenticationUtil.logger.error("exception occured in decrypt_encoded_data : {}".format(e))
            raise ApiException(
                status=0,
                reason="Failed to decrypt_encoded_data {}".format(e)
            )
